﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 루트
        /// </summary>
        private Octagon root = null;

        /// <summary>
        /// 크기 스케일
        /// </summary>
        private float sizeScale = (float)(1.0 / (2.0 * (1 + Math.Cos(Math.PI / 180 * 45))));

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            #region 이벤트를 설정한다.

            Load                                 += Form_Load;
            this.depthNumericUpDown.ValueChanged += depthNumericUpDown_ValueChanged;
            this.drawButton.Click                += drawButton_Click;
            this.canvasPictureBox.Paint          += canvasPictureBox_Paint;

            #endregion
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 폼 로드시 처리하기 - Form_Load(sender, e)

        /// <summary>
        /// 폼 로드시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Form_Load(object sender, EventArgs e)
        {
            DrawSierpinskiOctagon();
        }

        #endregion
        #region 깊이 숫자 UP/DOWN 값 변경시 처리하기 - depthNumericUpDown_ValueChanged(sender, e)

        /// <summary>
        /// 깊이 숫자 UP/DOWN 값 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void depthNumericUpDown_ValueChanged(object sender, EventArgs e)
        {
            DrawSierpinskiOctagon();
        }

        #endregion
        #region 그리기 버튼 클릭시 처리하기 - drawButton_Click(sender, e)

        /// <summary>
        /// 그리기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void drawButton_Click(object sender, EventArgs e)
        {
            DrawSierpinskiOctagon();
        }

        #endregion
        #region 캔버스 픽처 박스 페인트시 처리하기 - canvasPictureBox_Paint(sender, e)

        /// <summary>
        /// 캔버스 픽처 박스 페인트시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void canvasPictureBox_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.Clear(this.canvasPictureBox.BackColor);

            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;

            if(this.root == null)
            {
                return;
            }

            this.root.Draw(e.Graphics);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 8각형 포인트 배열 구하기 - GetOctagonPointArray(centerPoint, radius)

        /// <summary>
        /// 8각형 포인트 배열 구하기
        /// </summary>
        /// <param name="centerPoint">중심 포인트</param>
        /// <param name="radius">반지름</param>
        /// <returns>8각형 포인트 배열</returns>
        private PointF[] GetOctagonPointArray(PointF centerPoint, float radius)
        {
            PointF[] pointArray = new PointF[8];
            double   theta      = -Math.PI / 2.0;
            double   deltaTheta = 2.0 * Math.PI / 8.0;

            for(int i = 0; i < 8; i++)
            {
                pointArray[i] = new PointF
                (
                    centerPoint.X + (float)(radius * Math.Cos(theta)),
                    centerPoint.Y + (float)(radius * Math.Sin(theta))
                );

                theta += deltaTheta;
            }

            return pointArray;
        }

        #endregion
        #region 8각형 구하기 - GetOctagon(depth, centerPoint, radius)

        /// <summary>
        /// 8각형 구하기
        /// </summary>
        /// <param name="depth">깊이</param>
        /// <param name="centerPoint">중심 포인트</param>
        /// <param name="radius">반지름</param>
        /// <returns>8각형</returns>
        private Octagon GetOctagon(int depth, PointF centerPoint, float radius)
        {
            Octagon parent = new Octagon(GetOctagonPointArray(centerPoint, radius));

            if(depth > 0)
            {
                float d = radius - radius * this.sizeScale;

                PointF[] pointArray = GetOctagonPointArray(centerPoint, d);

                foreach(PointF point in pointArray)
                {
                    parent.ChildList.Add
                    (
                        GetOctagon
                        (
                            depth - 1,
                            point,
                            radius * this.sizeScale
                        )
                    );
                }
            }

            return parent;
        }

        #endregion
        #region 시어핀스키 8각형 그리기 - DrawSierpinskiOctagon()

        /// <summary>
        /// 시어핀스키 8각형 그리기
        /// </summary>
        private void DrawSierpinskiOctagon()
        {
            int depth = (int)this.depthNumericUpDown.Value;

            PointF centerPoint = new PointF
            (
                this.canvasPictureBox.ClientSize.Width  / 2,
                this.canvasPictureBox.ClientSize.Height / 2
            );

            float radius = (float)Math.Min(centerPoint.X, centerPoint.Y);

            radius -= 5;

            this.root = GetOctagon(depth, centerPoint, radius);

            this.canvasPictureBox.Refresh();
        }

        #endregion
    }
}